package io.zipcoder.crudapp;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.result.MockMvcResultMatchers;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import java.util.Arrays;
import java.util.List;

import static org.hamcrest.Matchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;


@RunWith(SpringRunner.class)
@SpringBootTest
public class CRUDApplicationTests {

	private MockMvc mockMvc;

	@Mock
	private PersonService personService;

	@InjectMocks
	private PersonController personController;

	@Before
	public void init(){
		MockitoAnnotations.initMocks(this);
		mockMvc = MockMvcBuilders
				.standaloneSetup(personController)
				.build();
	}

	@Test
	public void contextLoads() {
	}

	@Test
	public void test_get_all_success() throws Exception {
		List<Person> personList = Arrays.asList(
				new Person("Daenerys", "Targaryen"),
				new Person("John", "Snow"));

		when(personService.getAll()).thenReturn(personList);

		mockMvc.perform(get("/person"))
				.andExpect(status().isOk())
				.andExpect(content().contentType(MediaType.APPLICATION_JSON_UTF8_VALUE))
				.andExpect(MockMvcResultMatchers.jsonPath("$", hasSize(2)))

				.andExpect(MockMvcResultMatchers.jsonPath("$[0].firstName", is("Daenerys")))
				.andExpect(MockMvcResultMatchers.jsonPath("$[0].firstName", is("Targaryen")))
				.andExpect(MockMvcResultMatchers.jsonPath("$[0].id", is(1)))

				.andExpect(MockMvcResultMatchers.jsonPath("$[1].firstName", is("John")))
				.andExpect(MockMvcResultMatchers.jsonPath("$[0].firstName", is("Snow")))
				.andExpect(MockMvcResultMatchers.jsonPath("$[1].id", is(2)))
		;

		verify(personService, times(1)).getAll();
		verifyNoMoreInteractions(personService);
	}

}
